#!/bin/bash
# module 2
#This module uses the output of M01 (FITS_opt_M01.log), which has the form:
#column_1    -> complete spectra path name
#column_2-17 -> daospec option parameters (or sh lo le ri re mi ma ve fi sc cr wa sm ba fw)
#column_18   -> flag1 for fix (flag1=1) or not fix (flag1=0) fwhm analysis
#Ccolumn_19  -> spectrum type (U for UVES, G for GIRAFFE)
#and the output files of Daospec (*.daospec)
#It checks if the radial velocity dispersion of a spectra is consistent with the median of
#all the spectra (within 50%), and adds a flag2=1 if consistent, flag2=0 if not, distinguishing
#GIRAFFE from UVES.
#
#Then it extracts the result of output FWHM from the Daospec run of M01. Counts of many of them
#are within CONVf% of the input value (flag3=1). If less than 60% of them have flag3=0, rerun Daospec 
#with the output value as an input. It Does it MAXl times if needed, then if still not 60% of converged
#spectra exits the loop and leaves a flag3=0 for the non-converged spectra.
#
#The output file has the form:
#column_1-18 -> same as FITS_opt_M01.log, but fw may have been modified
#column_19   -> flag2 for radial velocity dispersion, =1 if consistent, 0 if not, 2 if not defined
#column_20   -> flag3 for FWHM measurement, =1 if output is within CONVf% of input, =0 if not; 2 if not defined
#column_21   -> spectrum type (U for UVES, G for GIRAFFE)

#CHECKING NEEDED FILE before going on

echo M02: ----------   module 02 is now running...   ----------

dir_fits=$WORK_DIR					#Working directory 
infile=`grep -w output01 pipe.opt | cut -d= -f2`	#Output file of module 1
majrevfile=`grep MAJfile.*= pipe.opt | cut -d= -f2`     #Major reviews file

if [ -z $infile ]; then
  echo "M02: input file (module 01 output) is not defined in pipe.opt ..."
  echo M02: ...setting it as output01.dat
  infile=output01.dat
fi
file_opt_m1=${dir_fits}/${infile}

if [ ! -f $file_opt_m1 ]; then
  echo M02: I cannot find the output file of module 1 $file_opt_m1.
  echo M02: Exiting the module.
  exit 1
fi

outfile=`grep -w output02 pipe.opt | cut -d= -f2`	#Output file of module 2
if [ -z $outfile ]; then
  echo M02: output file is not defined in pipe.opt ...
  echo ...setting it as output02.dat
  outfile=output02.dat
fi
file_opt_m2=${dir_fits}/${outfile}

if [ -z $majrevfile ]; then
  echo M02: major reviews file is not defined in pipe.opt ...
  majrevfile=majorrev.dat
  echo ...setting it as $majrevfile.
fi
majrevfile=${WORK_DIR}/$majrevfile

MAXl=`grep maxloops.*= pipe.opt | cut -d= -f2`	#setting the max number of loops
if [ -z $MAXl ]; then
  echo "M02: the max number of loop (maxloops) is not defined in pipe.opt ..."
  echo M02: ...setting it to 10
  MAXl=10
fi


CONVf=`grep convergence.*= pipe.opt | cut -d= -f2`	#setting the criterion for stating convergence on the FWHM
if [ -z $MAXl ]; then
  echo "M02: the convergence criterion on the FWHM (convergence) is not defined in pipe.opt ..."
  echo M02: ...setting it to 3%.
  CONVf=3
fi


FRACc=`grep fracc.*= pipe.opt | cut -d= -f2`	#setting the satisfying percentage of converged spectra
if [ -z $FRACc ]; then
  echo "M02: the percentage of converged spectra is not defined in pipe.opt ..."
  echo M02: ...setting it to 60%.
  FRACc=60
fi

cd $dir_fits
rm -f median*.temp

#CHECKING DUPLICATES IN inupt file and spectra in common with output file
checkduplicate=`grep -v \# $file_opt_m1 | awk 'x[$1]++' | wc -l`
if [ $checkduplicate -gt 0 ]; then echo M02: file $file_opt_m1 contains duplicated spectra. Please correct it.; exit 1; fi
if [ -f $file_opt_m2 ]; then
  checkduplicate=`cat $file_opt_m1 $file_opt_m2 | grep -v \# | awk 'x[$1]++' | wc -l` 
  if [ $checkduplicate -gt 0 ]; then 
    echo M02: file $file_opt_m1 and $file_opt_m2 have spectra in common. 
    echo M02: This may crash the pipeline. Please correct it.; exit 1;
  fi
fi

echo "#" >> $file_opt_m2 

#Read the output of M01:
list=(`grep -v \# $file_opt_m1 | awk '{print $1}'`)
stype=(`grep -v \# $file_opt_m1 | awk '{print $19}'`)

rm -f median*.temp	
trap 'echo M02: ...forced to exit.; exit 1' 2	#this command listen to the signal CTLR+C and force the script to stop with exit status 1

iu=0; ig=0; skip=0; index=0
for fits in ${list[@]}; do
  if [ ${stype[$index]} == 'U' ]; then
    listU[$iu]=${list[$index]}
    orU[$iu]=`grep $fits $file_opt_m1 | awk '{print $2}'`
    shU[$iu]=`grep $fits $file_opt_m1 | awk '{print $3}'`
    loU[$iu]=`grep $fits $file_opt_m1 | awk '{print $4}'`
    leU[$iu]=`grep $fits $file_opt_m1 | awk '{print $5}'`
    riU[$iu]=`grep $fits $file_opt_m1 | awk '{print $6}'`
    reU[$iu]=`grep $fits $file_opt_m1 | awk '{print $7}'`
    miU[$iu]=`grep $fits $file_opt_m1 | awk '{print $8}'`
    maU[$iu]=`grep $fits $file_opt_m1 | awk '{print $9}'`
    veU[$iu]=`grep $fits $file_opt_m1 | awk '{print $10}'`
    fiU[$iu]=`grep $fits $file_opt_m1 | awk '{print $11}'`
    scU[$iu]=`grep $fits $file_opt_m1 | awk '{print $12}'`
    crU[$iu]=`grep $fits $file_opt_m1 | awk '{print $13}'`
    waU[$iu]=`grep $fits $file_opt_m1 | awk '{print $14}'`
    smU[$iu]=`grep $fits $file_opt_m1 | awk '{print $15}'`
    baU[$iu]=`grep $fits $file_opt_m1 | awk '{print $16}'`
    fwU[$iu]=`grep $fits $file_opt_m1 | awk '{print $17}'`
    fwiU[$iu]=`grep $fits $file_opt_m1 | awk '{print $17}'`
    flag1U[$iu]=`grep $fits $file_opt_m1 | awk '{print $18}'`
    ((iu++))
  elif [ ${stype[$index]} == 'G' ]; then
    listG[$ig]=${list[$index]}
    orG[$ig]=`grep $fits $file_opt_m1 | awk '{print $2}'`
    shG[$ig]=`grep $fits $file_opt_m1 | awk '{print $3}'`
    loG[$ig]=`grep $fits $file_opt_m1 | awk '{print $4}'`
    leG[$ig]=`grep $fits $file_opt_m1 | awk '{print $5}'`
    riG[$ig]=`grep $fits $file_opt_m1 | awk '{print $6}'`
    reG[$ig]=`grep $fits $file_opt_m1 | awk '{print $7}'`
    miG[$ig]=`grep $fits $file_opt_m1 | awk '{print $8}'`
    maG[$ig]=`grep $fits $file_opt_m1 | awk '{print $9}'`
    veG[$ig]=`grep $fits $file_opt_m1 | awk '{print $10}'`
    fiG[$ig]=`grep $fits $file_opt_m1 | awk '{print $11}'`
    scG[$ig]=`grep $fits $file_opt_m1 | awk '{print $12}'`
    crG[$ig]=`grep $fits $file_opt_m1 | awk '{print $13}'`
    waG[$ig]=`grep $fits $file_opt_m1 | awk '{print $14}'`
    smG[$ig]=`grep $fits $file_opt_m1 | awk '{print $15}'`
    baG[$ig]=`grep $fits $file_opt_m1 | awk '{print $16}'`
    fwG[$ig]=`grep $fits $file_opt_m1 | awk '{print $17}'`
    fwiG[$ig]=`grep $fits $file_opt_m1 | awk '{print $17}'`
    flag1G[$ig]=`grep $fits $file_opt_m1 | awk '{print $18}'`
    ((ig++))
  else
    echo "M02: $fits is not GIRAFFE or UVES type. Skipping it."
    ((skip++))
  fi
  ((index++))
done
echo "M02: I found $iu UVES spectra and $ig GIRAFFE spectra in $file_opt_m1. $skip spectra will be skipped."

nbspectra=`awk -v PIP="$iu" -v PLU="$ig" 'BEGIN {print PIP+PLU}'`

if [ $nbspectra -eq 0 ]; then
  echo M02: No usable spectra in $file_opt_m1. Exiting the module.
  exit 1
fi

#Get all the RV dispersion and their median value

#If UVES spectra are found then playing with them
if [ $iu != 0 ]; then
  echo "M02: now playing with UVES spectra ... "
  index=0
  for fits in ${listU[@]}; do
    rootname=`echo $fits | sed 's/.fits//g'`
    dummy=`cat ${rootname}.daospec | grep Radial | awk '{print $7}'`
    if [ -z $dummy ]; then
      echo M02: Cannot find radial velocity dispersion for $fits in ${rootname}.daospec.
    else
      #To compute the median it uses only the data that are defined
      echo $dummy >> medianU.temp
    fi
    rvdisp[$index]=$dummy   
    ((index++))
  done
  
  medianrvdispU=`sort -n medianU.temp | awk ' { a[i++]=$1; } END { x=int((i+1)/2); if (x < (i+1)/2) print (a[x-1]+a[x])/2; else print a[x-1]; }'`
  echo M02: The median value of the Radial Velocity dispersion for UVES is $medianrvdispU.
  rm -f medianU.temp

  #Compare the individual RV dispersion with the median and set flag2
  #BTW the choice of putting flag2=1 if delta<10 and 0 if delta>10 
  #is perfectly arbitrary...
  index=0	
  for fits in ${listU[@]}; do
    if [ -n ${rvdisp[$index]} ]; then
      deltarvdisp=`awk -v PIP="${rvdisp[$index]}" -v PLU="$medianrvdispU" 'BEGIN {print PIP-PLU}'`
      conditiondeltarvdisp=`awk -v PIP="$deltarvdisp" 'BEGIN { if (PIP-10>0 || PIP+10<0 ) print 0; else print 1 }'` 
      flag2U[$index]=$conditiondeltarvdisp
      if [ $conditiondeltarvdisp -eq 0 ]; then
        sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M02: Flag2 set to 0. RV dispersion greater than expected with respect to the median value.|' $majrevfile
      fi
    else
      flag2U[$index]=2		#In case radial velocity dispersion is not defined
      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M02: Cannot find RV dispersion. Flag2 set to 2.|' $majrevfile
    fi
    #echo ${rvdisp[$index]} $deltarvdisp $conditiondeltavrdisp ${flag2[$index]}    #if you want to display the flags...
    ((index++))
  done
  echo "M02: Flag2 (=1 if radial velocity dispersion is consistent) was set for all UVES spectra."

  #Get all the FWHM from Daospec run 
  #and check if they are within 5% from the input value (flag3)
  index=0; nbflag1U=0; nbflag3U=0; notf=0
  for fits in ${listU[@]}; do     
    fwout[$index]=`echo ${fwU[$index]} | sed 's/fw=//g'`   
    if [ ${flag1U[$index]} -eq 1 ]; then   	# deal first with spectra for which you asked a fixed fw (they have flag1=1)
      flag3U[$index]=1         			# all spectra with flag1=1 automatically have flag3=1 as well
      ((nbflag3U++))           		
      ((nbflag1U++))				# here we count the number of flag1's
      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M02: imposed FIXED FWHM.|' $majrevfile
    elif [ ${flag1U[$index]} -eq 0 ]; then     	# for those with non-fixed fw, check the output fw before setting flag3
      rootname=`echo $fits | sed 's/.fits//g'`
      fwin=`grep fw= ${rootname}_M01.log | sed 's/fw=//g'`
      if [ -n $fwin ]; then
        deltafw=`awk -v PIP="$fwin" -v PLU="${fwout[$index]}" 'BEGIN {print (PIP-PLU)/PIP}'`
        conditiondeltafw=`awk -v PIP="$deltafw" -v PLU="$CONVf" 'BEGIN { if (PIP+PLU/100<0 || PIP-PLU/100>0 ) print 0; else print 1 }'`
        #echo M02: deltafw=$deltafw condition=$conditiondeltafw
        if [ $conditiondeltafw -eq 1 ]; then 
	  ((nbflag3U++))
	  sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM02: FWHM converged from M01.|' $majrevfile
	fi
        flag3U[$index]=$conditiondeltafw 
      else
        echo M02: Cannot find input FWHM for $fits in ${rootname}_M01.log. 
	flag3U[$index]=2			#In case fwhm is not defined
	((notf++))
        sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M02: Flag3 set to 2. Cannot find FWHM in '${rootname}'_M01.log.|' $majrevfile
      fi
    else
      echo M02: there is something wrong in flag1 for $fits. 
      flag3U[$index]=3				#In case strange values for flag1
      ((notf++))
      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M02: Flag3 set to 3. Strange value for Flag1.|' $majrevfile
    fi
    ((index++))
  done
  echo "M02: flag3 (about the convergence of the FWHM) was set for all UVES spectra. $notf spectra have no defined FWHM."

  if [ $nbflag1U -eq $iu ]; then 
    conditionfracofconvergedU=0
    percofconvergedU=100
  else
    fracofconvergedU=`awk -v PIP="$nbflag3U" -v PLU="$nbflag1U" -v TIZ="$iu" -v SEM="$notf" 'BEGIN {print (PIP-PLU)/(TIZ-PLU-SEM)}'`
    percofconvergedU=`awk -v PIP="$fracofconvergedU" 'BEGIN {print 100*PIP}'`
    conditionfracofconvergedU=`awk -v PIP="$fracofconvergedU" -v PLU="$FRACc" 'BEGIN { if (PIP<PLU/100 ) print 1; else print 0 }'`  #This is where you set the value of 60%
  fi
  echo M02: The fraction of spectra with flag3=1 is ${percofconvergedU}%.

  #rerun Daospec on the stars that have flag3=0, using fwout as new input.
  #this stops when FRACc% of the spectra are within CONVf%, or after MAXl loops

  loops=0
  maxloops=$MAXl  #arbitrary maximum number of loops, can be modified
  while [ $conditionfracofconvergedU -eq 1 ]; do   
    echo M02: Now trying to make input and output FWHM converge...
    while [ $loops -lt $maxloops ]; do
      index=0
      for fits in ${listU[@]}; do
  	path=`echo $fits | sed -e "s/\/[^\/]*$//"`
  	file=`echo $fits | sed -e "s/.*\///"`
  	rootname=`echo $fits | sed 's/.fits//g'`
  	if [ ${flag3U[$index]} -eq 0 ]; then
  	  cd $path
  	  fwin=${fwout[$index]}
  	  rm -f ${rootname}.daospec ${rootname}C.fits ${rootname}R.fits 
  	  echo -e ${orU[$index]} "\n"${shU[$index]} "\n"${loU[$index]} "\n"${leU[$index]} "\n"${riU[$index]} "\n"${reU[$index]} "\n"${miU[$index]} "\n"${maU[$index]} "\n"${veU[$index]} "\n"${fiU[$index]} "\n"${scU[$index]} "\n"${crU[$index]} "\n"${waU[$index]} "\n"${smU[$index]} "\n"${baU[$index]} "\n"fw"="${fwout[$index]} > daospec.opt
  	  echo M02: FWHM for spectrum $fits did not converge. Running Daospec with fw=${fwout[$index]}...
  	  echo "#DAOSPEC module 02 logfile" > ${rootname}_M02.log
  	  cat daospec.opt >> ${rootname}_M02.log
daospec << END >> ${rootname}_M02.log
  

$file
  
END
	  #get the new output fwhm
  	  fwout[$index]=`cat ${rootname}.daospec | grep FWHM | awk '{print $10}' | sed 's/,//g'`  
	  okfw=0
          if [[ "${fwout[$index]}" =~ ^[-0-9]+([.][0-9]+)?$ ]] ; then
            echo M02: ...done.			  
  	    deltafw=`awk -v PIP="$fwin" -v PLU="${fwout[$index]}" 'BEGIN {print (PIP-PLU)/PIP}'`	  
  	    conditiondeltafw=`awk -v PIP="$deltafw" -v PLU="$CONVf" 'BEGIN { if (PIP+PLU/100<0 || PIP-PLU/100>0 ) print 0; else print 1 }'`  #compare it with the one you just used as input
            fwU[$index]=fw=${fwout[$index]} 
  	    if [ $conditiondeltafw -eq 1 ]; then    
  	      flag3U[$index]=1			      #set flag3 to 1 if now the output is within 5% of the input
  	      ((nbflag3U++))
	      echo M02: Now FWHM converged for $fits \(${fwU[$index]}\).
	      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM02: FWHM converged after '$((loops+1))' iteration(s).|' $majrevfile
	    fi
	    okfw=1
	  else 
	    echo M02: Cannot find FWHM for $fits in ${rootname}.daospec.
	    flag3U[$index]=2			#In case fwhm is not defined
	    ((notf++))
            sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M02: NOT CONVERGED. Problem with DAOSPEC. Cannot find FWHM. Setting Flag3 to 2 and skipping further analysis.|' $majrevfile
	  fi
	  if [ ! -s ${rootname}.daospec -o $okfw -eq 0 ]; then
            echo "M02: Skipping to the next spectrum."
	    echo "M02: removing DAOSPEC output file:"
	    rm -fv ${rootname}.daospec
	  fi 
  	fi
  	((index++))
      #END of the FOR loop	
      done
      fracofconvergedU=`awk -v PIP="$nbflag3U" -v PLU="$nbflag1U" -v TIZ="$iu" -v SEM="$notf" 'BEGIN {print (PIP-PLU)/(TIZ-PLU-SEM)}'`
      conditionfracofconvergedU=`awk -v PIP="$fracofconvergedU" -v PLU="$FRACc" 'BEGIN { if (PIP<PLU/100 ) print 1; else print 0 }'`  
      ((loops++))
      echo M02: -------------- finished loop number $loops for UVES.
      if [ $conditionfracofconvergedU -eq 0 ]; then echo M02: we reached convergence for UVES.; loops=$maxloops; fi
      if [ $loops = $maxloops -a $conditionfracofconvergedU -eq 1 ]; then 
        echo M02: the maximum number of allowed loops was attained.
        conditionfracofconvergedU=0
      fi
    #END of the inner (second) while loop
    done
  #END of the first while loop
  done
  
  #Now write in a file the list of spectra, daospec parameters (with latest fwhm), and all FOUR flags
  #NB:here the printed fw is the input one of the last convergence, not the output 
  #(which can always be found in the output file if needed)
  index=0	
  for fits in ${listU[@]}; do
    echo -e "$fits ${orU[$index]} ${shU[$index]} ${loU[$index]} ${leU[$index]} ${riU[$index]} ${reU[$index]} ${miU[$index]} ${maU[$index]} ${veU[$index]} ${fiU[$index]} ${scU[$index]} ${crU[$index]} ${waU[$index]} ${smU[$index]} ${baU[$index]} ${fwU[$index]} ${flag1U[$index]} ${flag2U[$index]} ${flag3U[$index]}  U" >> $file_opt_m2
    _fi=`echo ${fwiU[$index]} | cut -d= -f2`
    _fo=`echo ${fwU[$index]} | cut -d= -f2`
    _temp=`awk -v PIP="$_fo" -v PLU="$_fi" 'function abs(x){return ((x < 0.0) ? -x : x)} BEGIN{FRAC=100*(PIP-PLU)/PLU;{printf "%i",abs(FRAC)}}'`
    _temp2=`awk -v PIP="$_fo" -v PLU="$_fi" 'BEGIN{if(PIP==PLU) print 1; else print 0}'`
    if [ $_temp -gt 70 ]; then
      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M02: FWHM largely (more than 70%) changed from input value passing from '$_fi' to '$_fo'|' $majrevfile
    elif [ $_temp2 -eq 0 ]; then
      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM02: FWHM changed from '$_fi' to '$_fo'|' $majrevfile
    fi	  
    ((index++))
  done
  echo M02: $nbflag3U UVES spectra on $iu have FWHM converged. 
  echo M02: For the ones with not defined FWHM \($notf spectra\), FWHM was left unchanged from the last well-ended analysis and are flagged with flag3=2.
fi

cd $dir_fits
rm -f median*.temp

#If GIRAFFE spectra are found then playing with them
if [ $ig != 0 ]; then
  echo "M02: now playing with GIRAFFE spectra ... "
  index=0
  for fits in ${listG[@]}; do
    rootname=`echo $fits | sed 's/.fits//g'`
    dummy=`cat ${rootname}.daospec | grep Radial | awk '{print $7}'`
    if [ -z $dummy ]; then
      echo M02: Cannot find radial velocity dispersion for $fits in ${rootname}.daospec
    else
      #To compute the median it uses only the data that are defined
      echo $dummy >> medianG.temp
    fi
    rvdisp[$index]=$dummy   
    ((index++))
  done
  
  medianrvdispG=`sort -n medianG.temp | awk ' { a[i++]=$1; } END { x=int((i+1)/2); if (x < (i+1)/2) print (a[x-1]+a[x])/2; else print a[x-1]; }'`
  echo M02: The median value of the Radial Velocity dispersion for GIRAFFE is $medianrvdispG.
  rm -f medianG.temp

  #Compare the individual RV dispersion with the median and set flag2
  #BTW the choice of putting flag2=1 if delta<10 and 0 if delta>10 
  #is perfectly arbitrary...
  index=0	
  for fits in ${listG[@]}; do
    if [ -n ${rvdisp[$index]} ]; then
      deltarvdisp=`awk -v PIP="${rvdisp[$index]}" -v PLU="$medianrvdispG" 'BEGIN {print PIP-PLU}'`
      conditiondeltarvdisp=`awk -v PIP="$deltarvdisp" 'BEGIN { if (PIP-10>0 || PIP+10<0 ) print 0; else print 1 }'` 	 
      flag2G[$index]=$conditiondeltarvdisp    
      if [ $conditiondeltarvdisp -eq 0 ]; then
        sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M02: Flag2 set to 0. RV dispersion greater than expected with respect to the median value.|' $majrevfile
      fi
    else
      flag2G[$index]=2		#In case radial velocity dispersion is not defined
      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M02: Cannot find RV dispersion. Flag2 set to 2.|' $majrevfile
    fi
    #echo ${rvdisp[$index]} $deltarvdisp $conditiondeltavrdisp ${flag2[$index]}    #if you want to display the flags...
    ((index++))
  done
  echo "M02: Flag2 (=1 if radial velocity dispersion is consistent) was set for all GIRAFFE spectra."

  #Get all the FWHM from Daospec run 
  #and check if they are within 5% from the input value (flag3)
  index=0; nbflag1G=0; nbflag3G=0; notf=0
  for fits in ${listG[@]}; do     
    fwout[$index]=`echo ${fwG[$index]} | sed 's/fw=//g'`   
    if [ ${flag1G[$index]} -eq 1 ]; then   	# deal first with spectra for which you asked a fixed fw (they have flag1=1)
      flag3G[$index]=1         			# all spectra with flag1=1 automatically have flag3=1 as well
      ((nbflag3G++))           		
      ((nbflag1G++))				# here we count the number of flag1's
      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M02: imposed FIXED FWHM.|' $majrevfile
    elif [ ${flag1G[$index]} -eq 0 ]; then     	# for those with non-fixed fw, check the output fw before setting flag3
      rootname=`echo $fits | sed 's/.fits//g'`
      fwin=`grep fw= ${rootname}_M01.log | sed 's/fw=//g'`
      if [ -n $fwin ]; then
        deltafw=`awk -v PIP="$fwin" -v PLU="${fwout[$index]}" 'BEGIN {print (PIP-PLU)/PIP}'`
        conditiondeltafw=`awk -v PIP="$deltafw" -v PLU="$CONVf" 'BEGIN { if (PIP+PLU/100<0 || PIP-PLU/100>0 ) print 0; else print 1 }'`
        #echo M02: deltafw=$deltafw condition=$conditiondeltafw
        if [ $conditiondeltafw -eq 1 ]; then 
	  ((nbflag3G++)) 
	  sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM02: FWHM converged from M01.|' $majrevfile
	fi 
        flag3G[$index]=$conditiondeltafw 
      else
        echo M02: Cannot find input FWHM for $fits in ${rootname}_M01.log. 
	flag3G[$index]=2			#In case fwhm is not defined
	((notf++))
        sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M02: Flag3 set to 2. Cannot find FWHM in '${rootname}'_M01.log.|' $majrevfile
      fi
    else
      echo M02: there is something wrong in flag1 for $fits. 
      flag3G[$index]=3				#In case strange values for flag1
      ((notf++))
      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M02: Flag3 set to 3. Strange value for Flag1.|' $majrevfile
    fi
    ((index++))
  done
  echo "M02: flag3 (about the convergence of the FWHM) was set for all GIRAFFE spectra."

  if [ $nbflag1G -eq $ig ]; then 
    conditionfracofconvergedG=0
    percofconvergedG=100
  else
    fracofconvergedG=`awk -v PIP="$nbflag3G" -v PLU="$nbflag1G" -v TIZ="$ig" -v SEM="$notf" 'BEGIN {print (PIP-PLU)/(TIZ-PLU-SEM)}'`
    percofconvergedG=`awk -v PIP="$fracofconvergedG" 'BEGIN {print 100*PIP}'`
    conditionfracofconvergedG=`awk -v PIP="$fracofconvergedG" -v PLU="$FRACc" 'BEGIN { if (PIP<PLU/100 ) print 1; else print 0 }'`  #This is where you set the value of 60%
  fi
  echo M02: The fraction of spectra with flag3=1 is ${percofconvergedG}%.

  #rerun Daospec on the stars that have flag3=0, using fwout as new input.
  #this stops when 60% of the spectra are within 5%, or after 5 loops

  loops=0
  maxloops=$MAXl  #arbitrary maximum number of loops, can be modified
  while [ $conditionfracofconvergedG -eq 1 ]; do   
    echo M02: Now trying to make input and output FWHM converge...
    while [ $loops -lt $maxloops ]; do
      index=0
      for fits in ${listG[@]}; do
  	path=`echo $fits | sed -e "s/\/[^\/]*$//"`
  	file=`echo $fits | sed -e "s/.*\///"`
  	rootname=`echo $fits | sed 's/.fits//g'`
  	if [ ${flag3G[$index]} -eq 0 ]; then
  	  cd $path
  	  fwin=${fwout[$index]}
  	  rm -f ${rootname}.daospec ${rootname}C.fits ${rootname}R.fits 
  	  echo -e ${orG[$index]} "\n"${shG[$index]} "\n"${loG[$index]} "\n"${leG[$index]} "\n"${riG[$index]} "\n"${reG[$index]} "\n"${miG[$index]} "\n"${maG[$index]} "\n"${veG[$index]} "\n"${fiG[$index]} "\n"${scG[$index]} "\n"${crG[$index]} "\n"${waG[$index]} "\n"${smG[$index]} "\n"${baG[$index]} "\n"fw"="${fwout[$index]} > daospec.opt
  	  echo M02: FWHM for spectrum $fits did not converge. Running Daospec with fw=${fwout[$index]}...
  	  echo "#DAOSPEC module 02 logfile" > ${rootname}_M02.log
  	  cat daospec.opt >> ${rootname}_M02.log
daospec << END >> ${rootname}_M02.log
  

$file
  
END
	  #get the new output fwhm
  	  fwout[$index]=`cat ${rootname}.daospec | grep FWHM | awk '{print $10}' | sed 's/,//g'`  
	  okfw=0
          if [[ "${fwout[$index]}" =~ ^[-0-9]+([.][0-9]+)?$ ]] ; then
            echo M02: ...done.		  
  	    deltafw=`awk -v PIP="$fwin" -v PLU="${fwout[$index]}" 'BEGIN {print (PIP-PLU)/PIP}'`	  
  	    conditiondeltafw=`awk -v PIP="$deltafw" -v PLU="$CONVf" 'BEGIN { if (PIP+PLU/100<0 || PIP-PLU/100>0 ) print 0; else print 1 }'`  #compare it with the one you just used as input
            fwG[$index]=fw=${fwout[$index]}
  	    if [ $conditiondeltafw -eq 1 ]; then    
  	      flag3G[$index]=1			      #set flag3 to 1 if now the output is within 5% of the input
  	      ((nbflag3G++))
	      echo M02: Now FWHM converged for $fits \(${fwU[$index]}\).
	      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM02: FWHM converged after '$((loops+1))' iteration(s).|' $majrevfile
  	    fi
	    okfw=1
	  else 
	    echo M02: Cannot find FWHM for $fits in ${rootname}.daospec.
	    flag3G[$index]=2			#In case fwhm is not defined
	    ((notf++))
            sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M02: NOT CONVERGED. Problem with DAOSPEC. Cannot find FWHM. Setting Flag3 to 2 and skipping further analysis.|' $majrevfile
	  fi
	  if [ ! -s ${rootname}.daospec -o $okfw -eq 0 ]; then
            echo "M02: Skipping to the next spectrum."
	    echo "M02: removing DAOSPEC output file:"
	    rm -fv ${rootname}.daospec
	  fi 
  	fi
  	((index++))
      #END of the FOR loop	
      done
      fracofconvergedG=`awk -v PIP="$nbflag3G" -v PLU="$nbflag1G" -v TIZ="$ig" -v SEM="$notf" 'BEGIN {print (PIP-PLU)/(TIZ-PLU-SEM)}'`
      conditionfracofconvergedG=`awk -v PIP="$fracofconvergedG" -v PLU="$FRACc" 'BEGIN { if (PIP<PLU/100 ) print 1; else print 0 }'`  
      ((loops++))
      echo M02: -------------- finished loop number $loops for GIRAFFE.
      if [ $conditionfracofconvergedG -eq 0 ]; then echo M02: we reached convergence for GIRAFFE.; loops=$maxloops; fi
      if [ $loops = $maxloops -a $conditionfracofconvergedG -eq 1 ]; then 
        echo M02: the maximum number of allowed loops was attained.
        conditionfracofconvergedG=0
      fi
    #END of the inner (second) while loop
    done
  #END of the first while loop
  done
  
  #Now write in a file the list of spectra, daospec parameters (with latest fwhm), and all FOUR flags
  #NB:here the printed fw is the input one of the last convergence, not the output 
  #(which can always be found in the output file if needed)
  index=0	
  for fits in ${listG[@]}; do
    echo -e "$fits ${orG[$index]} ${shG[$index]} ${loG[$index]} ${leG[$index]} ${riG[$index]} ${reG[$index]} ${miG[$index]} ${maG[$index]} ${veG[$index]} ${fiG[$index]} ${scG[$index]} ${crG[$index]} ${waG[$index]} ${smG[$index]} ${baG[$index]} ${fwG[$index]} ${flag1G[$index]} ${flag2G[$index]} ${flag3G[$index]}  G" >> $file_opt_m2
    _fi=`echo ${fwiG[$index]} | cut -d= -f2`
    _fo=`echo ${fwG[$index]} | cut -d= -f2`
    _temp=`awk -v PIP="$_fo" -v PLU="$_fi" 'function abs(x){return ((x < 0.0) ? -x : x)} BEGIN{FRAC=100*(PIP-PLU)/PLU;{printf "%i\n",abs(FRAC)}}'`
    _temp2=`awk -v PIP="$_fo" -v PLU="$_fi" 'BEGIN{if(PIP==PLU) print 1; else print 0}'`
    if [ $_temp -gt 70 ]; then
      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M02: FWHM largely (more than 70%) changed from input value passing from '$_fi' to '$_fo'|' $majrevfile
    elif [ $_temp2 -eq 0 ]; then
      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM02: FWHM changed from '$_fi' to '$_fo'|' $majrevfile
    fi	  
    ((index++))
  done
  echo M02: $nbflag3G GIRAFFE spectra on $ig have FWHM converged. 
  echo M02: For the ones with not defined FWHM \($notf spectra\), FWHM was left unchanged from the last well-ended analysis and are flagged with flag3=2.
fi

echo M02: The final parameters for the spectra were written in file \"$file_opt_m2\". This file is used by module3.sh.

cd $dir_fits
exit 0

